#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "elite.h"

#define DEBUG_MEMBER -1

#define POINT_MAX 100
#define UNFINISHED 20*60
#define MAX_TIME 59*60+59
#define MAX_FILE_SIZE 1000000

#define POINT_COLOUR "\"#FFFF80\""

char *mon[] = {"January", "February", "March", "April", "May", "June", "July",
			   "August", "September", "October", "November", "December"};

char *st_nam[20] = {"Dam",
					"Facility",
					"Runway",
					"Surface 1",
					"Bunker 1",
					"Silo",
					"Frigate",
					"Surface 2",
					"Bunker 2",
					"Statue",
					"Archives",
					"Streets",
					"Depot",
					"Train",
					"Jungle",
					"Control",
					"Caverns",
					"Cradle",
					"Aztec",
					"Egypt"};
char *st_fn[20] =  {"dam",
					"facility",
					"runway",
					"surface1",
					"bunker1",
					"silo",
					"frigate",
					"surface2",
					"bunker2",
					"statue",
					"archives",
					"streets",
					"depot",
					"train",
					"jungle",
					"control",
					"caverns",
					"cradle",
					"aztec",
					"egypt"};
char *di_nam[3] = {"Agent", "Secret Agent", "00 Agent"};
int sanity[20][3] = {
	{1, 1, 1},			/* Dam */
	{1, 1, 1},			/* Facility */
	{1, 1, 1},			/* Runway */
	{1, 1, 1},			/* Surface 1 */
	{1, 1, 1},			/* Bunker 1 */
	{1, 1, 1},			/* Silo */
	{1, 1, 1},			/* Frigate */
	{1, 1, 1},			/* Surface 2 */
	{1, 1, 1},			/* Bunker 2 */
	{1, 1, 1},			/* Statue */
	{1, 1, 1},			/* Archives */
	{1, 1, 1},			/* Streets */
	{1, 1, 1},			/* Depot */
	{1, 1, 1},			/* Train */
	{1, 1, 1},			/* Jungle */
	{1, 1, 1},			/* Control */
	{1, 1, 1},			/* Caverns */
	{1, 1, 1},			/* Cradle */
	{1, 1, 1},			/* Aztec */
	{3, 6, 9}			/* Egypt */
};

char tab_buf[MAX_FILE_SIZE+1];
int tab_size;

void writetime( FILE *fil, int n ) {
	if( n >= 3600 ) {
		fprintf( fil, "%d:%02d:%02d", n/3600, (n/60)%60, n%60 );
	} else {
		fprintf( fil, "%2d:%02d", (n/60)%60, n%60 );
	}
}

void writeupdatetime( FILE *fil ) {
	time_t tt;
	struct tm *t;

	time(&tt);
	t = localtime(&tt);
	fprintf( fil, "%s %d, %d ", mon[t->tm_mon], t->tm_mday, t->tm_year+1900 );
	fprintf( fil, "%d:%02d ", (t->tm_hour+11)%12+1, t->tm_min );
	fprintf( fil, "%cM EST", (t->tm_hour>=12)?'P':'A' );
}

void domodcommands( int n ) {
	char filename[50];
	FILE *fil;
	char *match = "!--GEE ";
	int ml = strlen(match);
	char ch;
	int i;

	strcpy(filename, "lc\\");
	strcat(filename, el[n].fn);
	strcat(filename, ".html");
	fil = fopen( filename, "r" );
	if( !fil ) return;

	while( !feof(fil) ) {
		do {
			ch = fgetc(fil);
		} while( ch != '<' && !feof(fil) );
		tab_size = 0;
		while( ch != '>' && !feof(fil) && tab_size < MAX_FILE_SIZE ) {
			tab_buf[tab_size++] = ch = fgetc(fil);
		}
		if( feof(fil) || tab_size == MAX_FILE_SIZE ) break;
		if( strncmp( match, tab_buf, ml ) ) continue;
		if( strncmp( "--!>", tab_buf+tab_size-4, 4 ) ) continue;
		tab_size -= 4;
		tab_buf[tab_size] = 0;

		/* INVALID command */
		if( !strncmp( "INVALID ", tab_buf+ml, 8 ) ) {
			for( i = 0; i < NumEl; i++ ) {
				if( !stricmp( tab_buf+ml+8, el[i].name ) )
					el[i].invalid = 1;
			}
		}
	}
}

void readfiletable( char *fn ) {
	char filename[50];
	char ch;
	int pos;
	FILE *fil;
	char *match1 = "<table", *match2 = "</table>";
	int m1l, m2l;
	int ct=0;

	m1l = strlen(match1); m2l = strlen(match2);

	tab_size = 0;
	strcpy(filename, fn);
	fil = fopen( filename, "r" );
	if( !fil ) goto invalid_file;

	while( !feof(fil) ) {
		pos = tab_size;
		for(;;) {
			ch = fgetc(fil);
			if( tab_size == MAX_FILE_SIZE ) goto invalid_file;
			if( feof(fil) ) break;
			tab_buf[tab_size++] = tolower(ch);
			if( !strncmp(tab_buf+(tab_size-m1l), match1, m1l) ) break;
		}
		tab_size = pos;

		for(;;) {
			ch = fgetc(fil);
			if( tab_size == MAX_FILE_SIZE ) goto invalid_file;
			if( feof(fil) ) break;
			tab_buf[tab_size++] = ch;
			if( ch == '<' ) {
				pos = tab_size;
				for(;;) {
					ch = fgetc(fil);
					if( feof(fil) ) break;
					tab_buf[tab_size++] = tolower(ch);
					if( ch == '>' ) break;
				}
				tab_size = pos-1;
				if( !strncmp(tab_buf+tab_size, match2, m2l) ) break;

				ch = ' ';
				if( !strncmp(tab_buf+tab_size, "</td>", 5) ||
					!strncmp(tab_buf+tab_size, "<td", 3) ) {
					ch = '\n';
				}
				if( tab_size ) {
					if( !isspace(tab_buf[tab_size-1]) )
						tab_size++;
					if( tab_buf[tab_size-1] != '\n' )
						tab_buf[tab_size-1] = ch;
				}
			} else if( isspace(ch) ) {
				if( tab_size == 1 || isspace(tab_buf[tab_size-2]) )
					tab_size--;
				else
					tab_buf[tab_size-1] = ' ';
			} else if( (ch == ';' && tab_size > 6 &&
					 !strncmp(tab_buf+tab_size-6, "&nbsp;", 6)) ) {
				tab_size -= 6;
				if( tab_size > 0 && !isspace(tab_buf[tab_size-1]) )
					tab_buf[tab_size++] = ' ';
			} else if( ch == '*' ) {
				tab_size--;
				if( tab_buf[tab_size-1] == '/' )
					tab_size--;
			} else if( ch == '(' ) {
				pos = tab_size;
				for(;;) {
					ch = fgetc(fil);
					if( feof(fil) ) break;
					tab_buf[tab_size++] = tolower(ch);
					if( ch == ')' ) break;
				}
				tab_size = pos-1;
			}
		}
	}
	fclose(fil);
	if( tab_size && isspace(tab_buf[tab_size-1]) ) tab_size--;
	tab_buf[tab_size] = 0;
	return;

invalid_file:
	tab_buf[0] = 0;
	tab_size = 0;
	return;
}

char *tab_st_str[20][4] = {
	{"dam"},
	{"facility"},
	{"runway"},
	{"surface 1", "surface one"},
	{"bunker 1", "bunker one"},
	{"silo"},
	{"frigate"},
	{"surface 2", "surface two"},
	{"bunker 2", "bunker two"},
	{"statue"},
	{"archives"},
	{"streets"},
	{"depot"},
	{"train"},
	{"jungle"},
	{"control"},
	{"caverns", "cavern"},
	{"cradle"},
	{"aztec"},
	{"egypt"}
};
char *tab_inc_str[10] = {"n/a", "??", "--", "incomplete", "-:--",
						 0, 0, 0, 0, 0};

int readrecs( int n ) {
	int i, j, st = 0;
	char *cp, *cp2, *cp3;
	int n1, n2;
	int col;

	readfiletable( el[n].fn );
	if( n == DEBUG_MEMBER )
		printf( "%s:\n", el[n].name );
	for( cp = strtok(tab_buf, "\n"); st < 20 && cp; cp = strtok(0, "\n") ) {
		for( j = 0; j < 4; j++ )
			if( tab_st_str[st][j] && strstr(cp, tab_st_str[st][j]) )
				break;
		if( j == 4 ) continue;

		if( n == DEBUG_MEMBER )
			printf( "%s\n", cp );
		cp = strstr(cp, tab_st_str[st][j]) + strlen(tab_st_str[st][j]);
		for( j = 0, cp2 = cp; *cp2 && j < 3; j++ ) {
			for(; *cp2; cp2++ ) {
				for( i = 0; i < 10; i++ )
					if( tab_inc_str[i] &&
						!strncmp(cp2, tab_inc_str[i], strlen(tab_inc_str[i])) )
						break;
				if( i < 10 ) {
					el[n].rec[j][st] = UNFINISHED;
					for(; *cp2 && *cp2 != ' '; cp2++);
					goto done_time;
				}
				if( (*cp2 >= '0' && *cp2 <= '9') ||
					(*cp2 == ':' && *(cp2+1) >= '0' && *(cp2+1) <= '9') )
					break;
			}
			if( !*cp2 ) break;
			n1 = n2 = col = 0;
			while( *cp2 >= '0' && *cp2 <= '9' ) {
				n2 = n2*10 + (*cp2-'0');
				cp2++;
			}
			while( *cp2 == ':' && *(cp2+1) == ':' ) cp2++;
			if( *cp2 == ':' ) {
				col = 1;
				cp2++;
				n1 = n2;
				n2 = 0;
				while( *cp2 >= '0' && *cp2 <= '9' ) {
					n2 = n2*10 + (*cp2-'0');
					cp2++;
				}
			}
			if( *cp2  == ':' ) for(; *cp2 != ' '; cp2++);
			for(; *cp2 == ' '; cp2++);
			if( !col && *cp2 == 'm' ) {
				for(; *cp2; cp2++ )
					if( *cp2 >= '0' && *cp2 <= '9' ) break;
				cp3 = cp2;
				n1 = n2;
				n2 = 0;
				while( *cp2 >= '0' && *cp2 <= '9' ) {
					n2 = n2*10 + (*cp2-'0');
					cp2++;
				}
				for(; *cp2 == ' '; cp2++);
				if( *cp2 == 'm' ) {
					n2 = 0;
					cp2 = cp3;
				}
			}
			while( *cp2 == '/' || *cp2 == '(' ||
					((*cp2 == '-' || *cp2 == '+') &&
					 ((*(cp2+1) >= '0' && *(cp2+1) <= '9') ||
					  (*(cp2+1) == ':' && *(cp2+2) >= '0' && *(cp2+2) <= '9'))
					)) {
				do {cp2++;} while( *cp2 && *cp2 == ' ' );
				do {cp2++;} while( *cp2 && *cp2 != ' ' );
				do {cp2++;} while( *cp2 && *cp2 == ' ' );
			}
			el[n].rec[j][st] = n1*60+n2;

/* Final sanity check */
			if( el[n].rec[j][st] < sanity[st][j] )
				el[n].rec[j][st] = UNFINISHED;
			if( el[n].rec[j][st] >= MAX_TIME )
				el[n].rec[j][st] = UNFINISHED;

done_time:
			n=n;
		}
		for( ; j < 3; j++ ) {
			el[n].rec[j][st] = UNFINISHED;
		}
		if( n == DEBUG_MEMBER ) {
			printf( "   " );
			writetime( stdout, el[n].rec[0][st] );
			printf( "  " );
			writetime( stdout, el[n].rec[1][st] );
			printf( "  " );
			writetime( stdout, el[n].rec[2][st] );
			printf( "\n" );
		}
		st++;
	}
	if( st < 20 ) {
		if( st != 0 )
			printf( "Error reading stage '%s' from %s.\n",
					st_nam[st], el[n].name );
		return 0;
	}
	for( j = 0; j < 3; j++ ) {
		el[n].rec[j][21] = 0;
		for( i = 0; i < 20; i++ ) el[n].rec[j][21] += el[n].rec[j][i];
	}
	return 1;
}

void backuprecs( int n ) {
	char filename[50];
	FILE *fil;
	char ch;
	int j, st;

	strcpy(filename, "lc\\");
	strcat(filename, el[n].fn);
	strcat(filename, ".bak");
	fil = fopen( filename, "w" );
	if( !fil ) {
		printf( "ERROR BACKING UP %s\n", el[n].name );
		exit(0);
	}
	for( st = 0; st < 20; st++ ) {
		fprintf( fil, "%2d.", st+1 );
		for( j = 0; j < 3; j++ )
			fprintf( fil, " %d", el[n].rec[j][st] );
		fprintf( fil, "\n" );
	}
	fclose(fil);
}

void readbackuprecs( int n ) {
	char filename[50];
	FILE *fil;
	char ch;
	int i, j, st;

	strcpy(filename, "lc\\");
	strcat(filename, el[n].fn);
	strcat(filename, ".bak");
	fil = fopen( filename, "r" );
	if( !fil ) {
		printf( "ERROR READING BACKUP FOR %s.\n", el[n].name );
		exit(0);
	}
	for( st = 0; st < 20; st++ ) {
		fscanf( fil, "%*d." );
		for( j = 0; j < 3; j++ ) {
			fscanf( fil, " %d", &el[n].rec[j][st] );

			/* Final sanity check */
			if( el[n].rec[j][st] < sanity[st][j] )
				el[n].rec[j][st] = UNFINISHED;
		}

	}
	fclose(fil);
	for( j = 0; j < 3; j++ ) {
		el[n].rec[j][21] = 0;
		for( i = 0; i < 20; i++ ) el[n].rec[j][21] += el[n].rec[j][i];
	}
}

char ch[] = "-----";
char cycle[10000];
int cyclelen = -1;
int cyclep = -1, cyclen = -1;
FILE *ifil, *ofil;

void get() {
	ch[0] = ch[1];
	ch[1] = ch[2];
	ch[2] = ch[3];
	ch[3] = ch[4];
	if( cyclep != -1 ) {
		ch[4] = cycle[cyclep++];
		if( cyclep == cyclelen ) {
			cyclep = 0;
			if( ++cyclen == NumEl ) {
				cyclep = -1;
				cyclelen = -1;
				cyclen = -1;
			}
		}
	} else {
		ch[4] = fgetc(ifil);
		if( cyclelen != -1 ) cycle[cyclelen++] = ch[4];
	}
}

void get2() {
	get();
	if( !feof(ifil) ) {
		if( ch[1] != '*' && ch[2] == '*' && ch[3] == '*' && ch[4] != '*' ) {
			fputc( '*', ofil );
			fputc( '*', ofil );
		}
		if( ch[2] != '*' && ch[3] == '*' && ch[4] != '*' ) {
			fputc( '*', ofil );
		}
		if( ch[1] != '&' && ch[2] == '&' && ch[3] == '&' && ch[4] != '&' ) {
			fputc( '&', ofil );
			fputc( '&', ofil );
		}
		if( ch[2] != '&' && ch[3] == '&' && ch[4] != '&' ) {
			fputc( '&', ofil );
		}
		if( ch[4] != '*' && ch[4] != '&' ) {
			fputc( ch[4], ofil );
		}
	}
	if( !strcmp( ch+2, "&&&" ) ) {
		if( cyclelen == -1 )
			cyclelen = 0;
		else {
			cyclelen -= 3;
			cyclep = 0;
			cyclen = 1;
		}
	}
}

typedef struct {
	int time;
	int pos;
	int points;
	int el;
} Rank;

int rcmp( void *a, void *b ) {
	if( ((Rank *)a)->time != ((Rank *)b)->time )
		return ((Rank *)a)->time-((Rank *)b)->time;
	return stricmp(el[((Rank *)a)->el].name, el[((Rank *)b)->el].name);
}

void rank( Rank *ranks ) {
	int i, x;

	for( i = 0; i < NumEl; i++ ) ranks[i].el = i;
	qsort( ranks, NumEl, sizeof(Rank), rcmp );
	for( i = 0, x = 0; i < NumEl; i++ ) {
		if( i == 0 || ranks[i-1].time != ranks[i].time ) x = i;
		ranks[i].pos = x+1;
		if( x == 0 ) ranks[i].points = POINT_MAX;
		if( x == 1 ) ranks[i].points = POINT_MAX-3;
		if( x > 1 && x < POINT_MAX-3 ) ranks[i].points = POINT_MAX-3-x;
		if( x >= POINT_MAX-3 ) ranks[i].points = 0;
	}
}

int find(char *nam) {
	char name[1000];
	int i, j;

	strcpy(name, nam);
	if( !strcmp( nam, "Darrel Simpson" ) )
		strcpy( name, "Darrell Simpson" );
	if( !strcmp( nam, "Dirk olson" ) )
		strcpy( name, "Dirk Olson" );
	if( !strcmp( nam, "Brian Novel" ) )
		strcpy( name, "Brian Novell" );
	if( !strcmp( nam, "Carl Micehsen" ) )
		strcpy( name, "Carl Michelsen" );
	if( !strcmp( nam, "Garrett Sellati." ) )
		strcpy( name, "Garrett Sellati" );
	if( !strcmp( nam, "Keven McBeth" ) )
		strcpy( name, "Kevin McBeth" );
	if( !strcmp( nam, "M.P Farrier" ) )
		strcpy( name, "M.P. Farrier" );
	if( !strcmp( nam, "Martin Kendall=" ) )
		strcpy( name, "Martin Kendall" );
	if( !strcmp( nam, "Mathew Yakobina" ) )
		strcpy( name, "Matthew Yakobina" );
	if( !strcmp( nam, "Pete Lobczowski" ) )
		strcpy( name, "Pete Lobsczowski" );
	if( !strcmp( nam, "Shwan Johnson" ) )
		strcpy( name, "Shawn Johnson" );
	for( i = 0; i < NumEl; i++ )
		if( !strcmp( name, el[i].name ) ) return i;
	strcpy( el[NumEl].name, name );
	strcpy( el[NumEl].url, "---" );
	strcpy( el[NumEl].email, "???" );
	for( j = 0; j < 3; j++ )
		for( i = 0; i < 20; i++ )
			el[NumEl].rec[j][i] = UNFINISHED;
	return NumEl++;
}

main() {
	int i, j, k, x, y, z, n, ch, st, dif;
	int tot[3];
	char fn[100];
	char *p1, *p2;

	for( st = 0; st < 20; st++ ) {
		strcpy(fn, "html\\");
		strcat(fn, st_fn[st]);
		strcat(fn, ".htm");
		readfiletable(fn);
		p1 = tab_buf;
		for( i = 0; i < 5; i++ )
			p1 = strchr(p1, '\n')+1;
		dif = 0;
		for(;;) {
			ch = 0;
			sscanf( p1, "%d:%d", &x, &y );
			p1 = strchr(p1, '/')+1;
			if( isspace(*p1) ) p1++;
			p2 = p1;
			for(;;) {
				while( !isspace(*p2) ) p2++;
				if( *p2 == '\n' || !*p2 ) {
					ch = 1;
					break;
				}
				if( *(p2+1) >= '0' && *(p2+1) <= '9' ) break;
				p2++;
			}
			*p2 = 0;
			n = find(p1);
			if( el[n].rec[dif][st] == UNFINISHED ||
				x*60+y < el[n].rec[dif][st] )
				el[n].rec[dif][st] = x*60+y;
			p1 = p2+1;
			dif += ch;
			if( dif == 3 ) break;
		}
	}
	qsort(el, NumEl, sizeof(Elite), elcmp);
	for( i = 0; i < NumEl; i++ ) {
		sprintf( el[i].fn, "%d", i+1 );
	}
	for( i = 0; i < NumEl; i++ ) {
		printf( "%s\n", el[i].fn );
		printf( "%s\n", el[i].name );
		printf( "%s\n", el[i].url );
		printf( "%s\n", el[i].email );
		printf( "\n" );
		backuprecs(i);
	}
}
